package rice.pastry.join;

import rice.pastry.*;
import rice.pastry.messaging.*;
import rice.pastry.security.*;

import java.io.*;
import java.util.*;

/**
 * Request for the join protocols on the local node to join the overlay.
 * 
 * @version $Id: InitiateJoin.java 2902 2006-01-10 15:23:05Z jeffh $
 * 
 * @author Andrew Ladd
 */

public class InitiateJoin extends Message implements Serializable {
  private NodeHandle[] handle;

  /**
   * Constructor.
   * 
   * @param nh the node handle that the join will begin from.
   */

  public InitiateJoin(NodeHandle nh) {
    super(new JoinAddress());
	handle = new NodeHandle[1];
    handle[0] = nh;
  }


  public InitiateJoin(NodeHandle[] nh) {
    super(new JoinAddress());

    handle = nh;
  }

  /**
   * Constructor.
   * 
   * @param jh a handle of the node trying to join the network.
   * @param stamp the timestamp
   * 
   * @param nh the node handle that the join will begin from.
   */

  public InitiateJoin(Date stamp, NodeHandle[] nh) {
    super(new JoinAddress(), stamp);

    handle = nh;
  }

  /**
   * Constructor.
   * 
   * @param jh a handle of the node trying to join the network.
   * @param cred the credentials
   * 
   * @param nh the node handle that the join will begin from.
   */

  public InitiateJoin(Credentials cred, NodeHandle[] nh) {
    super(new JoinAddress(), cred);

    handle = nh;
  }

  /**
   * Constructor.
   * 
   * @param jh a handle of the node trying to join the network.
   * @param cred the credentials
   * @param stamp the timestamp
   * 
   * @param nh the node handle that the join will begin from.
   */

  public InitiateJoin(Credentials cred, Date stamp, NodeHandle[] nh) {
    super(new JoinAddress(), cred, stamp);

    handle = nh;
  }

  /**
   * Gets the handle for the join.
   * 
   * Gets the first non-dead handle for the join.
   * 
   * @return the handle.
   */

  public NodeHandle getHandle() {
    for (int i = 0; i < handle.length; i++) {
      if (handle[i].isAlive()) return handle[i];
    }
    return null;
  }
}

